import numpy as np
from shogun import AccuracyMeasure
from shogun import AveragedPerceptron
from shogun import BinaryLabels
from shogun import CSVFile
from shogun import RealFeatures

f_feats_train = CSVFile("../../data/classifier_binary_2d_linear_features_train.dat")
f_feats_test = CSVFile("../../data/classifier_binary_2d_linear_features_test.dat")
f_labels_train = CSVFile("../../data/classifier_binary_2d_linear_labels_train.dat")
f_labels_test = CSVFile("../../data/classifier_binary_2d_linear_labels_test.dat")

#![create_features]
features_train = RealFeatures(f_feats_train)
features_test = RealFeatures(f_feats_test)
labels_train = BinaryLabels(f_labels_train)
labels_test = BinaryLabels(f_labels_test)
#![create_features]

#![set_parameters]
learn_rate = 1.0
max_iter = 1000
perceptron = AveragedPerceptron(features_train, labels_train)
perceptron.set_learn_rate(learn_rate)
perceptron.set_max_iter(max_iter)
#![set_parameters]

#![train_and_apply]
perceptron.train()
perceptron.set_features(features_test)
labels_predict = perceptron.apply()
#![train_and_apply]

#![extract_weights]
weights = perceptron.get_w()
bias = perceptron.get_bias()
#![extract_weights]

#![evaluate_accuracy]
eval = AccuracyMeasure()
accuracy = eval.evaluate(labels_predict, labels_test)
#![evaluate_accuracy]
