import numpy as np
from shogun import CSVFile
from shogun import KNN
from shogun import LMNN
from shogun import MulticlassAccuracy
from shogun import MulticlassLabels
from shogun import RealFeatures

f_feats_train = CSVFile("../../data/classifier_4class_2d_linear_features_train.dat")
f_feats_test = CSVFile("../../data/classifier_4class_2d_linear_features_test.dat")
f_labels_train = CSVFile("../../data/classifier_4class_2d_linear_labels_train.dat")
f_labels_test = CSVFile("../../data/classifier_4class_2d_linear_labels_test.dat")

#![create_features]
features_train = RealFeatures(f_feats_train)
features_test = RealFeatures(f_feats_test)
labels_train = MulticlassLabels(f_labels_train)
labels_test = MulticlassLabels(f_labels_test)
#![create_features]

#![create_instance]
k = 3
lmnn = LMNN(features_train, labels_train, k)
#![create_instance]

#![train_metric]
lmnn.train()
lmnn_distance = lmnn.get_distance()
#![train_metric]

#![train_and_apply]
knn = KNN(k, lmnn_distance, labels_train)
knn.train()
labels_predict = knn.apply_multiclass(features_test)
#![train_and_apply]

#![evaluate_accuracy]	
acc = MulticlassAccuracy()
accuracy = acc.evaluate(labels_predict, labels_test)
#![evaluate_accuracy]

# additional integration testing variables
output = labels_predict.get_labels()

